/*
 * XdccDownloader - DccSendReceived.java - Copyright © 2013 David Roden
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.pterodactylus.irc.event;

import java.net.InetAddress;

import net.pterodactylus.irc.Connection;
import net.pterodactylus.irc.Source;

/**
 * Notifies a listener that a DCC SEND message was received.
 *
 * @author <a href="mailto:bombe@pterodactylus.net">David ‘Bombe’ Roden</a>
 */
public class DccSendReceived extends AbstractConnectionEvent {

	/** The source of the DCC SEND. */
	private final Source source;

	/** The name of the file being offered. */
	private final String filename;

	/** The IP address of the source. */
	private final InetAddress inetAddress;

	/** The port number of the source. */
	private final int port;

	/** The filesize. */
	private final long filesize;

	/**
	 * Creates a new DCC SEND received event.
	 *
	 * @param connection
	 * 		The connetion the event occured on
	 * @param source
	 * 		The source offering the file
	 * @param filename
	 * 		The name of the file being offered
	 * @param inetAddress
	 * 		The IP address of the source
	 * @param port
	 * 		The port number of the source
	 * @param filesize
	 * 		The size of the file being offered ({@code -1} for unknown size)
	 */
	public DccSendReceived(Connection connection, Source source, String filename, InetAddress inetAddress, int port, long filesize) {
		super(connection);
		this.source = source;
		this.filename = filename;
		this.inetAddress = inetAddress;
		this.port = port;
		this.filesize = filesize;
	}

	//
	// ACCESSORS
	//

	/**
	 * Returns the source offering the file.
	 *
	 * @return The source offering the file
	 */
	public Source source() {
		return source;
	}

	/**
	 * Returns the name of the file being offered.
	 *
	 * @return The name of the file being offered
	 */
	public String filename() {
		return filename;
	}

	/**
	 * Returns the size of the file being offered. If the size is not known, {@code
	 * -1} is returned.
	 *
	 * @return The size of the file being offered, or {@code -1} if the size is not
	 *         known
	 */
	public long filesize() {
		return filesize;
	}

	/**
	 * Return the IP address of the source.
	 *
	 * @return The IP address of the source
	 */
	public InetAddress inetAddress() {
		return inetAddress;
	}

	/**
	 * Returns the port number of the source.
	 *
	 * @return The port number of the source
	 */
	public int port() {
		return port;
	}

}
