/*
 * XdccDownloader - SearchCommand.java - Copyright © 2013 David Roden
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.pterodactylus.xdcc.ui.stdin;

import static com.google.common.collect.Lists.newArrayList;
import static java.util.Arrays.asList;
import static java.util.regex.Pattern.CASE_INSENSITIVE;
import static java.util.regex.Pattern.UNICODE_CASE;
import static java.util.regex.Pattern.compile;
import static java.util.stream.Collectors.toList;

import java.io.IOException;
import java.io.Writer;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.regex.Pattern;

import net.pterodactylus.xdcc.core.Core;
import net.pterodactylus.xdcc.data.Bot;
import net.pterodactylus.xdcc.data.Pack;

/**
 * Command that searches all {@link Pack}s of all {@link Bot}s for files
 * matching the search parameters.
 *
 * @author <a href="mailto:bombe@pterodactylus.net">David ‘Bombe’ Roden</a>
 */
public class SearchCommand implements Command {

	/** The core to operate on. */
	private final Core core;

	/**
	 * Creates a new search command.
	 *
	 * @param core
	 * 		The core to operate on
	 */
	public SearchCommand(Core core) {
		this.core = core;
	}

	//
	// COMMAND METHODS
	//

	@Override
	public String getName() {
		return "search";
	}

	@Override
	public Collection<String> getAliases() {
		return asList("find", "locate");
	}

	@Override
	public State execute(State state, List<String> parameters, Writer outputWriter) throws IOException {
		List<Result> lastResult = newArrayList();
		List<SearchParameter> searchParameters = parameters.stream().map(SearchParameter::from).collect(toList());
		for (Bot bot : newArrayList(core.bots())) {
			for (Pack pack : newArrayList(bot)) {
				if (searchParameters.stream().allMatch((parameter) -> parameter.matches(pack.name()))) {
					lastResult.add(new Result(core, bot, pack));
				}
			}
		}
		Collections.sort(lastResult);
		int counter = 0;
		for (Result result : lastResult) {
			outputWriter.write(String.format("[%d] %s (%s) from %s (#%s) on %s\n", counter++, result.pack().name(), result.pack().size(), result.bot().name(), result.pack().id(), result.bot().network().name()));
		}
		outputWriter.write("End of Search.\n");
		outputWriter.flush();
		return state.setLastResults(lastResult);
	}

	private static class SearchParameter {

		private final boolean exclude;
		private final Pattern pattern;

		private SearchParameter(boolean exclude, Pattern pattern) {
			this.exclude = exclude;
			this.pattern = pattern;
		}

		public boolean matches(String filename) {
			return pattern.matcher(filename).find() && !exclude;
		}

		public static SearchParameter from(String parameter) {
			boolean exclude = parameter.startsWith("-");
			Pattern pattern = compile(exclude ? parameter.substring(1) : parameter, CASE_INSENSITIVE | UNICODE_CASE);
			return new SearchParameter(exclude, pattern);
		}

	}

}
